lc      = 5e-2; // characteristic element size
arcSize = Pi/4.5; // less than pi
holeSize_1 = .15;
holeSize_2 = .10;


Function CheeseHole
  p1 = newp; Point(p1) = {x,  y,  z,  lc} ;
  p2 = newp; Point(p2) = {x+r,y,  z,  lc} ;
  p3 = newp; Point(p3) = {x,  y+r,z,  lc} ;
  p4 = newp; Point(p4) = {x,  y,  z+r,lc} ;
  p5 = newp; Point(p5) = {x-r,y,  z,  lc} ;
  p6 = newp; Point(p6) = {x,  y-r,z,  lc} ;
  p7 = newp; Point(p7) = {x,  y,  z-r,lc} ;

  c1 = newreg; Circle(c1) = {p2,p1,p7}; c2 = newreg; Circle(c2) = {p7,p1,p5};
  c3 = newreg; Circle(c3) = {p5,p1,p4}; c4 = newreg; Circle(c4) = {p4,p1,p2};
  c5 = newreg; Circle(c5) = {p2,p1,p3}; c6 = newreg; Circle(c6) = {p3,p1,p5};
  c7 = newreg; Circle(c7) = {p5,p1,p6}; c8 = newreg; Circle(c8) = {p6,p1,p2};
  c9 = newreg; Circle(c9) = {p7,p1,p3}; c10 = newreg; Circle(c10) = {p3,p1,p4};
  c11 = newreg; Circle(c11) = {p4,p1,p6}; c12 = newreg; Circle(c12) = {p6,p1,p7};

  // We need non-plane surfaces to define the spherical holes. Here we
  // use ruled surfaces, which can have 3 or 4 sides:

  l1 = newreg; Line Loop(l1) = {c5,c10,c4};   Ruled Surface(newreg) = {l1};
  l2 = newreg; Line Loop(l2) = {c9,-c5,c1};   Ruled Surface(newreg) = {l2};
  l3 = newreg; Line Loop(l3) = {c12,-c8,-c1}; Ruled Surface(newreg) = {l3};
  l4 = newreg; Line Loop(l4) = {c8,-c4,c11};  Ruled Surface(newreg) = {l4};
  l5 = newreg; Line Loop(l5) = {-c10,c6,c3};  Ruled Surface(newreg) = {l5};
  l6 = newreg; Line Loop(l6) = {-c11,-c3,c7}; Ruled Surface(newreg) = {l6};
  l7 = newreg; Line Loop(l7) = {-c2,-c7,-c12};Ruled Surface(newreg) = {l7};
  l8 = newreg; Line Loop(l8) = {-c6,-c9,c2};  Ruled Surface(newreg) = {l8};

  // We then store the surface loops identification numbers in a list
  // for later reference (we will need these to define the final
  // volume):

  cheeseHole[t] = newreg ;

  Surface Loop(cheeseHole[t]) = {l8+1,l5+1,l1+1,l2+1,l3+1,l7+1,l6+1,l4+1};

  // thehole = newreg ;
  // Volume(thehole) = cheeseHole[t] ;

Return


Point(1) = {0,0,0,lc};
Point(2) = {1,0,0,lc};
Point(3) = {Cos(arcSize),Sin(arcSize),0,lc};
Point(4) = {0,0,1,lc};
Point(5) = {1,0,1,lc};
Point(6) = {Cos(arcSize),Sin(arcSize),1,lc};

Circle(1) = {2,1,3};
Line(2)   = {3,1};
Line(3)   = {1,2};
Line Loop(11) = {1,2,3}; 
Plane Surface(100)  = {11}; // bottom surface

// making the first hole
Point(7)  = {.8,            0, 1, lc};
Point(8)  = {.8+holeSize_1, 0, 1, lc};
Point(9)  = {.8-holeSize_1, 0, 1, lc};
Point(10)  = {.8, holeSize_1, 1, lc};
Point(11)  = {.8, 0, 1-holeSize_1};
Circle(4) = {8,7,11};
Circle(5) = {11,7,9}; // two circle pieces, front face
Circle(6) = {8,7,10};
Circle(7) = {10,7,9}; // two circle pieces, top face

Line(8)  = {1, 4};
Line(9)  = {4, 9};
Line(10) = {8, 5};
Line(11) = {5, 2};
Line Loop(12) = {-3, 8, 9, -5, -4, 10, 11}; // outer loop for front surface

Point(12)  = {.3, 0, .3, lc};
Point(13)  = {.3+holeSize_2, 0, .3, lc};
Point(14)  = {.3, 0, .3+holeSize_2, lc};
Point(15)  = {.3-holeSize_2, 0, .3, lc};
Point(16)  = {.3, 0, .3-holeSize_2, lc};
Point(17)  = {.3, holeSize_2, .3, lc};
Circle(15) = {13,12,14};
Circle(16) = {14,12,15};
Circle(17) = {15,12,16};
Circle(18) = {16,12,13};
Circle(19) = {17,12,13};
Circle(20) = {17,12,14};
Circle(21) = {17,12,15};
Circle(22) = {17,12,16};
Line Loop(17) = {15, 16, 17, 18};
Plane Surface(101) = {12, 17}; // front surface

Line(12)      = {4, 6};
Circle(13)    = {6, 4, 5};
Line(14)      = {3, 6};
// outBack[]  = Extrude{0,0,1} { Line{2}; }; // back surface
// outRight[] = Extrude{0,0,1} { Line{1}; }; // right surface
Line Loop(13) =  {2, 8, 12, -14};
Line Loop(14) =  {1, 14, 13, 11};
Plane Surface(102) = {13}; // back surface
Ruled Surface(103) = {14}; // right surface

Line Loop(15) = {6, 7, -9, 12, 13, -10};
Plane Surface(104) = {15}; // top surface

Line Loop(16) = {6,7,-5,-4};
Ruled Surface(105)  = {16}; // hole 1 surface

Line Loop(18) = {19, 15, -20}; Ruled Surface(106) = {18};
Line Loop(19) = {20, 16, -21}; Ruled Surface(107) = {19};
Line Loop(20) = {21, 17, -22}; Ruled Surface(108) = {20};
Line Loop(21) = {22, 18, -19}; Ruled Surface(109) = {21};

Surface Loop(1001) = {100, 101, 102, 103, 104, 105, 106, 107, 108, 109}; // outer surface loop

t = 0; x=.5; y=.25; z=.2; r=.09;
Call CheeseHole;
t = 1; x=.65; y=.36; z=.6; r=.13;
Call CheeseHole;
t = 2; x=.23; y=.11; z=.81; r=.03;
Call CheeseHole;
t = 3; x=.7; y=.23; z=.31; r=.05;
Call CheeseHole;

Volume(1001) = {1001, cheeseHole[0], cheeseHole[1], cheeseHole[2], cheeseHole[3] };

